<?php
/*
 * Plugin Name: CardsShield Gateway Authorize.net
 * Plugin URI:
 * Description: CardsShield Gateway Authorize.net
 * Author: CardsShield
 * Author URI: https://cardsshield.com
 * Version: 1.0.11
 *
 /*
 * This action hook registers our PHP class as a WooCommerce payment gateway
 */

if (!defined('ABSPATH')) {
    exit;
}
register_activation_hook(__FILE__, 'mecom_gateway_authorize_install');

require_once(plugin_dir_path(__FILE__) . 'utils.php');


if (!class_exists('CSAuthorizeUpdateChecker')) {

    class CSAuthorizeUpdateChecker
    {

        public $plugin_slug;
        public $version;
        public $cache_key;
        public $cache_allowed;

        public function __construct()
        {

            $this->plugin_slug = plugin_basename(__DIR__);
            $this->version = OPT_MECOM_AUTHORIZE_VERSION;
            $this->cache_key = 'cs_authorize_update_checker';
            $this->cache_allowed = true;

            add_filter('plugins_api', [$this, 'info'], 20, 3);
            add_filter('site_transient_update_plugins', [$this, 'update']);
            add_action('upgrader_process_complete', [$this, 'purge'], 10, 2);

        }

        public function request()
        {

            $remote = get_transient($this->cache_key);

            if (false === $remote || !$this->cache_allowed) {

                $remote = wp_remote_get(
                    'https://update.cardsshield.com/cs_authorize/info.php?site=' . urlencode(get_site_url()),
                    [
                        'timeout' => 10,
                        'headers' => [
                            'Accept' => 'application/json'
                        ]
                    ]
                );

                if (
                    is_wp_error($remote)
                    || 200 !== wp_remote_retrieve_response_code($remote)
                    || empty(wp_remote_retrieve_body($remote))
                ) {
                    return false;
                }

                set_transient($this->cache_key, $remote, DAY_IN_SECONDS);

            }

            $remote = json_decode(wp_remote_retrieve_body($remote));

            return $remote;

        }


        function info($res, $action, $args)
        {

            // do nothing if you're not getting plugin information right now
            if ('plugin_information' !== $action) {
                return $res;
            }

            // do nothing if it is not our plugin
            if ($this->plugin_slug !== $args->slug) {
                return $res;
            }

            // get updates
            $remote = $this->request();

            if (!$remote) {
                return $res;
            }

            $res = new stdClass();

            $res->name = $remote->name;
            $res->slug = $remote->slug;
            $res->version = $remote->version;
            $res->tested = $remote->tested;
            $res->requires = $remote->requires;
            $res->author = $remote->author;
            $res->author_profile = $remote->author_profile;
            $res->download_link = $remote->download_url;
            $res->trunk = $remote->download_url;
            $res->requires_php = $remote->requires_php;
            $res->last_updated = $remote->last_updated;

            $res->sections = [
                'description' => $remote->sections->description,
                'installation' => $remote->sections->installation,
                'changelog' => $remote->sections->changelog
            ];

            if (!empty($remote->banners)) {
                $res->banners = [
                    'low' => $remote->banners->low,
                    'high' => $remote->banners->high
                ];
            }

            return $res;

        }

        public function update($transient)
        {

            if (empty($transient->checked)) {
                return $transient;
            }

            $remote = $this->request();

            if (
                $remote
                && version_compare($this->version, $remote->version, '<')
                && version_compare($remote->requires, get_bloginfo('version'), '<')
                && version_compare($remote->requires_php, PHP_VERSION, '<')
            ) {
                $res = new stdClass();
                $res->slug = $this->plugin_slug;
                $res->plugin = plugin_basename(__FILE__); // misha-update-plugin/misha-update-plugin.php
                $res->new_version = $remote->version;
                $res->tested = $remote->tested;
                $res->package = $remote->download_url;

                $transient->response[$res->plugin] = $res;

            }

            return $transient;

        }

        public function purge($upgrader_object, $options)
        {

            if (
                $this->cache_allowed
                && 'update' === $options['action']
                && 'plugin' === $options['type']
            ) {
                // just clean the cache when new plugin version is installed
                delete_transient($this->cache_key);
            }

        }


    }

    new CSAuthorizeUpdateChecker();

}

//Cron
add_filter('cron_schedules', 'mecom_add_authorize_cron_interval');
if (!wp_next_scheduled('mecom_gateway_authorize_rotation')) {
    wp_schedule_event(time(), 'one_minute', 'mecom_gateway_authorize_rotation');
}

add_action('mecom_gateway_authorize_rotation', 'mecom_authorize_rotation_checker');
if (!wp_next_scheduled('mecom_gateway_authorize_daily')) {
    wp_schedule_event(strtotime('23:59:59'), 'daily', 'mecom_gateway_authorize_daily');
}
add_action('mecom_gateway_authorize_daily', 'mecom_gateway_authorize_daily_process');

function mecom_gateway_authorize_daily_process()
{
    // Reset paid amount
    $rotationMethod = get_option(OPT_MECOM_AUTHORIZE_ROTATION_METHOD, MECOM_AUTHORIZE_BY_TIME);
    if ($rotationMethod === MECOM_AUTHORIZE_BY_AMOUNT) {
        resetPaidAmountAuthorize();
    }
}

add_action('plugins_loaded', 'mecom_add_gateway_authorize_init');

function mecom_add_authorize_cron_interval($schedules)
{
    $schedules['one_minute'] = array(
        'interval' => 60,
        'display' => esc_html__('Every minute'),
    );

    return $schedules;
}

function mecom_gateway_authorize_install()
{
//    delete_option('mecom_gateway_authorize_version');
//    add_option('mecom_gateway_authorize_version', uniqid());
}

function mecom_authorize_rotation_checker()
{

    $rotationMethod = get_option(OPT_MECOM_AUTHORIZE_ROTATION_METHOD, MECOM_AUTHORIZE_BY_TIME);
    if ($rotationMethod != MECOM_AUTHORIZE_BY_TIME) {
        return;
    }
    // Auto Switching Proxy
    $proxies = get_option(Opt_Mecom_Authorize_Proxies, []);
    if (empty($proxies)) {
        return;
    }
    $activatedProxy = get_option(Opt_Mecom_Authorize_Activated_Proxy, null);
    // if only has 1 proxy, don't rotate
    if (count($proxies) === 1 && isset($activatedProxy['id']) && $activatedProxy['id'] === $proxies[0]['id']) {
        return;
    }
    $lastActivatedTimestamp = get_option(OPT_MECOM_AUTHORIZE_CURRENT_ROTATION_VALUE, 0);

    $hasDeletedActivateProxy = false;

    if ($lastActivatedTimestamp != 0 && !empty($activatedProxy)) {
        $currentTimestamp = time();
        $timeStampSinceLastActivated = $currentTimestamp - $lastActivatedTimestamp;
        // How many minutes?
        $minutes = $timeStampSinceLastActivated / 60;

        // Need to rotate
        if ($minutes > floatval($activatedProxy["timestamp"])) {
            $hasDeletedActivateProxy = true;
            for ($i = 0; $i < count($proxies); $i++) {
                $proxy = $proxies[$i];
                if ($proxy["id"] == $activatedProxy["id"]) {
                    // The last one => active the proxy at 0 index
                    if ($i + 1 == count($proxies)) {
                        $needToActivateProxy = $proxies[0];
                    } else {
                        // Active the next one
                        $needToActivateProxy = $proxies[$i + 1];
                    }
                    update_option(Opt_Mecom_Authorize_Activated_Proxy, $needToActivateProxy, true);
                    update_option(OPT_MECOM_AUTHORIZE_CURRENT_ROTATION_VALUE, time(), true);
                    logAuthorizeRotation(MECOM_AUTHORIZE_BY_TIME, $needToActivateProxy, "Auto");
                    $hasDeletedActivateProxy = false;
                    break;
                }
            }
        }
    }

    if (empty($activatedProxy) || $hasDeletedActivateProxy) {
        update_option(Opt_Mecom_Authorize_Activated_Proxy, $proxies[0], true);
        update_option(OPT_MECOM_AUTHORIZE_CURRENT_ROTATION_VALUE, time(), true);
        logAuthorizeRotation(MECOM_AUTHORIZE_BY_TIME, $proxies[0], "Auto");
    }
}


function mecom_add_gateway_authorize_init()
{

    if (is_admin()) {
        require_once("m-ecom-authorize-options.php");
    }

    if (!class_exists('WC_MEcom_Authorize')) :

        class WC_MEcom_Authorize
        {

            /**
             * @var Singleton The reference the *Singleton* instance of this class
             */
            private static $instance;

            /**
             * Returns the *Singleton* instance of this class.
             *
             * @return Singleton The *Singleton* instance.
             */
            public static function get_instance()
            {
                if (null === self::$instance) {
                    self::$instance = new self();
                }
                return self::$instance;
            }

            /**
             * Private clone method to prevent cloning of the instance of the
             * *Singleton* instance.
             *
             * @return void
             */
            private function __clone()
            {
            }

            /**
             * Private unserialize method to prevent unserializing of the *Singleton*
             * instance.
             *
             * @return void
             */
            public function __wakeup()
            {
            }

            /**
             * Protected constructor to prevent creating a new instance of the
             * *Singleton* via the `new` operator from outside of this class.
             */
            private function __construct()
            {
                $this->init();
            }

            private function init()
            {
                add_filter('woocommerce_payment_gateways', array($this, 'add_gateways'));
                if (is_admin()) {
                    add_filter(
                        'plugin_action_links_' . plugin_basename(__FILE__),
                        array($this, 'add_settings_link')
                    );
                }
                add_filter('woocommerce_available_payment_gateways', array($this, 'check_cs_authorize_payment_gateways'), 10, 1);
            }

            public function check_cs_authorize_payment_gateways($gateways)
            {
                if (!is_checkout() && !defined('WOOCOMMERCE_CHECKOUT')) {
                    return $gateways;
                }

                $rotationMethod = get_option(OPT_MECOM_AUTHORIZE_ROTATION_METHOD, MECOM_AUTHORIZE_BY_TIME);
                global $woocommerce;

                if ($rotationMethod == MECOM_AUTHORIZE_BY_AMOUNT) {
                    $carTotal = WC()->cart->get_total(false);
                    if (!hasPayableProxyAuthorize($carTotal)) {
                        unset($gateways['mecom_authorize']);
                    }
                }

                return $gateways;
            }

            /**
             * Add the gateways to WooCommerce.
             *
             * @since 1.0.0
             * @version 4.0.0
             */
            public function add_gateways($gateways)
            {
                $gateways[] = 'WC_MEcom_Gateway_Authorize'; // your class name is here
                return $gateways;
            }

            public function add_settings_link($links)
            {
                $settings = array(
                    'settings' => sprintf(
                        '<a href="%s">%s</a>',
                        admin_url('admin.php?page=wc-settings&tab=checkout&section=mecom_authorize'),
                        'Settings'
                    )
                );
                return array_merge($settings, $links);
            }
        }

        WC_MEcom_Authorize::get_instance();

        if (!class_exists('WC_Payment_Gateway')) {
            return;
        }

        class WC_MEcom_Gateway_Authorize extends WC_Payment_Gateway
        {
            /**
             * Whether or not logging is enabled
             *`
             * @var bool
             */
            public static $log_enabled = false;
            public static $mecom_authorize_is_inited = false;

            /**
             * Logger instance
             *
             * @var WC_Logger
             */
            public static $log = false;

            /**
             * @var string
             */
            public $productTitleSetting = 'last_word';
            
            public $userDefineProductTitle = 'ME';

            /**
             * @var string
             */
            public $invoice_prefix;

            /**
             * Class constructor, more about it in Step 3
             */
            public function __construct()
            {
                $this->id = 'mecom_authorize'; // payment gateway plugin ID
                $this->icon = ''; // URL of the icon that will be displayed on checkout page near your gateway name
                $this->has_fields = true; // in case you need a custom credit card form
                $this->method_title = 'CardsShield Gateway Authorize.net';
                $this->order_button_text = __('Place order', 'woocommerce');
                $this->method_description = 'CardsShield Gateway Authorize.net'; // will be displayed on the options page
                $this->invoice_prefix = $this->get_option('invoice_prefix');
                $this->productTitleSetting    = $this->get_option( 'product_title_setting' );
                $this->userDefineProductTitle = $this->get_option( 'user_define_product_title' );
                // gateways can support subscriptions, refunds, saved payment methods,
                // but in this tutorial we begin with simple payments
                $this->supports = array(
                    'products',
                    'refunds',
                );

                // Load the settings.
                $this->init_form_fields();
                $this->init_settings();

                // Define user set variables.
                $this->enabled = $this->get_option('enabled');
                $this->title = $this->get_option('title');
                $this->description = $this->get_option('description');
                $this->payment_notes = $this->get_option('payment_notes');
                $this->description .= ' ' . $this->payment_notes;
                $this->description = trim($this->description);
                if(!self::$mecom_authorize_is_inited) {
                    self::$mecom_authorize_is_inited = true;
                    add_filter( 'manage_edit-shop_order_columns', [ $this, 'add_mecom_shield_url' ], 10, 1 );
                    add_filter( 'manage_shop_order_posts_custom_column', [ $this, 'add_mecom_order_values' ], 10, 2 );
                }
                add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
                add_action('wp_enqueue_scripts', array($this, 'payment_scripts'));
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
                // add_action( 'woocommerce_order_status_on-hold_to_processing', array( $this, 'capture_payment' ) );
                // add_action( 'woocommerce_order_status_on-hold_to_completed', array( $this, 'capture_payment' ) );
                // add_action( 'woocommerce_api_mecom-process-payment', array( $this, 'webhook' ) );

                if (!$this->is_valid_for_use()) {
                    $this->enabled = 'no';
                }
            }
            
            public function add_mecom_shield_url( $columns ) {
                $statusColumnPos = array_search( 'order_status', array_keys( $columns ), true );
                $insertPos       = false === $statusColumnPos ? count( $columns ) : $statusColumnPos + 1;
        
                return array_merge(
                    array_slice( $columns, 0, $insertPos ),
                    [
                        'mecom_shield_url' => 'Shield URL',
                    ],
                    array_slice( $columns, $insertPos )
                );
            }
            
            public function add_mecom_order_values($column, $wc_order_id) {
                $this->add_mecom_shield_url_value($column, $wc_order_id);
            }
            
            public function add_mecom_shield_url_value( $column, $wc_order_id ) {
                if ( 'mecom_shield_url' != $column ) {
                    return;
                }
                $wc_order = wc_get_order( $wc_order_id );
        
                if ( ! is_a( $wc_order, \WC_Order::class )) {
                    return;
                }
                if ($wc_order->get_payment_method() === 'mecom_authorize') {            
                    echo '[Authorize.net] '. get_post_meta($wc_order_id, METAKEY_AUTHORIZE_PROXY_URL, true);
                }
            }

            /**
             * Plugin options, we deal with it in Step 3 too
             */
            public function init_form_fields()
            {
                $this->form_fields = array(
                    'enabled' => array(
                        'title' => 'Enable/Disable',
                        'label' => 'Enable CardsShield Gateway',
                        'type' => 'checkbox',
                        'description' => '',
                        'default' => 'no'
                    ),
                    'title' => array(
                        'title' => 'Title',
                        'type' => 'text',
                        'description' => '--------------------------------------------------------------',
                        'default' => 'Card',
                        'desc_tip' => false,
                    ),
                    'payment_notes' => array(
                        'title' => 'Payment notes',
                        'type' => 'textarea',
                        'description' => __('Payment notes are limited to 100 characters, cannot use the special characters.', 'woocommerce-gateway-authorize'),
                        'default' => '',
                        'desc_tip' => true,
                    ),
                    'statement_descriptor' => array(
                        'title' => 'Statement Descriptor',
                        'type' => 'text',
                        'description' => __('Statement descriptors are limited to 22 characters, cannot use the special characters >, <, ", \, \', *, and must not consist solely of numbers. This will appear on your customer\'s statement in capital letters.', 'woocommerce-gateway-authorize'),
                        'default' => '',
                        'desc_tip' => true,
                        'required' => true,
                    ),
                    'invoice_prefix' => array(
                        'title' => __('Invoice Prefix', 'woocommerce-gateway-authorize-express-checkout'),
                        'type' => 'text',
                        'description' => __('Please enter a prefix for your invoice numbers.', 'woocommerce-gateway-authorize-express-checkout'),
                        'default' => 'WC-',
                        'desc_tip' => true,
                        'required' => true,
                    ),
                    'product_title_setting'     => [
                        'title'       => 'Overwrite product title',
                        'type'        => 'select',
                        'description' => '',
                        'default'     => 'last_word',
                        'desc_tip'    => false,
                        'options'     => [
                            'last_word'     => 'Use the last word',
                            'user_define'   => 'User define',
                            'keep_original' => 'Keep the original (Not recommended)'
                        ]
                    ],
                    'user_define_product_title' => [
                        'title'       => 'User define title',
                        'type'        => 'text',
                        'description' => 'This will be appeared on Authorize.net transaction as product title, when overwrite product title is "User define"',
                        'default'     => 'ME',
                        'desc_tip'    => true,
                    ],
                    'config_proxies_button' => [
                        'id' => 'config_proxies_button',
                        'type' => 'config_proxies_button',
                        'title' => __('Config Shields', 'custom_authorize'),
                    ],
                    'card_icons' => array(
                        'type' => 'multiselect',
                        'title' => 'Accepted Payment Methods',
                        'class' => 'wc-enhanced-select',
                        'default' => array('visa', 'mastercard', 'american_express', 'discover', 'diners', 'jcb'),
                        'options' => array(
                            'visa' => 'Visa',
                            'paypal' => 'Paypal',
                            'mastercard' => 'MasterCard',
                            'jcb' => 'JCB',
                            'discover' => 'Discover',
                            'diners' => 'Diners Club',
                            'american_express' => 'American Express',
                        ),
                        'desc_tip'    => true,
                        'description' => 'The selected icons will show customers which credit card brands you accept.',
                    ),
                    'custom_css' => [
                        'title'   => 'Custom Authorize.net icon css',
                        'type'    => 'textarea',
                        'default' => '/*
    .ccicon {
        top: 46% !important;
    }
*/',
                        'css' => 'width: 400px; min-height: 110px; resize: both;',
                    ],
                    'custom_card_icon_css' => [
                        'title'   => 'Custom Authorize.net payment icon css',
                        'type'    => 'textarea',
                        'default' => '/*
.mecom-authorize-payment-icon {
    width: 50px;
}
*/',
                        'css' => 'width: 400px; min-height: 110px; resize: both;',
                    ]
                );
            }

            /**
             * Screen button Field
             */
            public function generate_config_proxies_button_html($key, $value)
            {
                ?>
                <tr valign="top">
                    <td colspan="2" class="forminp forminp-<?php echo sanitize_title($value['type']) ?>">
                        <a href="<?php echo admin_url('admin.php?page=mecom-gateway-authorize'); ?>"
                           class="button"><?php _e('Config Shields', 'custom_authorize'); ?></a>
                    </td>
                </tr>
                <?php
            }


            /**
             * Check if this gateway is enabled and available in the user's country.
             *
             * @return bool
             */
            public function is_valid_for_use()
            {
                return in_array(
                    get_woocommerce_currency(),
                    apply_filters(
                        'woocommerce_authorize_supported_currencies',
                        array('AUD', 'BRL', 'CAD', 'MXN', 'NZD', 'HKD', 'SGD', 'USD', 'EUR', 'JPY', 'TRY', 'NOK', 'CZK', 'DKK', 'HUF', 'ILS', 'MYR', 'PHP', 'PLN', 'SEK', 'CHF', 'TWD', 'THB', 'GBP', 'RMB', 'RUB', 'INR')
                    ),
                    true
                );
            }

            /**
             * Get_icon function.
             *
             * @return string
             * @version 4.0.0
             * @since 1.0.0
             */
            public function get_icon()
            {
                 $icons = $this->get_option('card_icons');
                 $icons_str = '';
                 if(is_array($icons)) {
                   foreach ($icons as $icon) {
                        $icons_str = '<img class="mecom-authorize-payment-icon" src="' . plugins_url('/assets/images/icons/' . $icon . '.svg', __FILE__) . '" style="width: 50px; padding-top: 2px; margin-right: 8px"/>' . $icons_str;                     
                     }  
                 }
                 
                 $icons_str .= '<style type="text/css">' . $this->get_option('custom_card_icon_css') . '</style>';
                 return apply_filters('woocommerce_gateway_icon', $icons_str, $this->id);
            }

            /**
             * Load admin scripts.
             *
             * @since 3.3.0
             */
            public function admin_scripts()
            {
                $screen = get_current_screen();
                $screen_id = $screen ? $screen->id : '';

                if ('woocommerce_page_wc-settings' !== $screen_id) {
                    return;
                }

                $suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';

                // wp_enqueue_script( 'woocommerce_authorize_admin', WC()->plugin_url() . '/includes/gateways/authorize/assets/js/authorize-admin' . $suffix . '.js', array(), WC_VERSION, true );
            }

            /**
             * You will need it if you want your custom credit card form, Step 4 is about it
             */
            public function payment_fields()
            {
                global $woocommerce;
                // ok, let's display some description before the payment form
                if ($this->description) {
                    // display the description with <p> tags etc.
                    echo wpautop(wp_kses_post($this->description));
                }
                $nextProxyUrl = WC()->session->get('mecom-authorize-proxy-active-url');
                if ($nextProxyUrl) {
                    // // I will echo() the form, but you can close PHP tags and print it directly in HTML
                    echo '<fieldset id="wc-' . esc_attr($this->id) . '-cc-form" class="wc-credit-card-form wc-payment-form" style="background:transparent; min-width: 250px; width: 100%">';

                    // // Add this action hook if you want your custom payment gateway to support it
                    do_action('woocommerce_credit_card_form_start', $this->id);
                    echo '<style>' . $this->get_option('custom_css') . '</style>';
                    echo '<script src="' . plugins_url('/assets/js/card_cc2.js', __FILE__) . '"></script>';
                    ?>
                    <iframe id="authorize-payment-area" scrolling="no"
                            src="<?= $nextProxyUrl . '?mecom-authorize-get-payment-form=1&need-decide-testmode=1' ?>"
                            height="0" frameBorder="0" style="width: 100%;"></iframe>
                    <div class="authorize-form-container">
                        <div class="field-container authorize_cardnumber_container">
                            <label for="authorize_cardnumber">Card Number</label>
                            <input id="authorize_cardnumber" type="text" pattern="[0-9]*" inputmode="numeric"
                                   name="mecom_authorize_ccNumber" placeholder="•••• •••• •••• ••••">
                            <svg id="authorize_ccicon" class="ccicon" width="750" height="471" viewBox="0 0 750 471"
                                 version="1.1" xmlns="http://www.w3.org/2000/svg"
                                 xmlns:xlink="http://www.w3.org/1999/xlink">
                            </svg>
                        </div>
                        <div class="field-container authorize_expirationdate_container half-left">
                            <label for="authorize_expirationdate">Expiry Date</label>
                            <input id="authorize_expirationdate" type="text" pattern="[0-9]*" inputmode="numeric"
                                   name="mecom_authorize_ccExpirationDate" placeholder="MM/YY">
                        </div>
                        <div class="field-container authorize_securitycode_container half-right">
                            <label for="authorize_securitycode">CVC</label>
                            <input id="authorize_securitycode" type="text" pattern="[0-9]*" inputmode="numeric"
                                   name="mecom_authorize_CCV" placeholder="CVC">
                        </div>
                    </div>
                    <?php
                    echo '<script src="' . plugins_url('/assets/js/libs/imask.min.js', __FILE__) . '"></script>';
                    echo '<script src="' . plugins_url('/assets/card/card.js', __FILE__) . '?ver=1.1"></script>';
                    do_action('woocommerce_credit_card_form_end', $this->id);

                    echo '<div class="clear"></div></fieldset>';
                } else {
                    ?>
                    <div>We cannot accept any payments right now. Please comeback to try tomorrow or select other
                        payment methods if available.
                    </div>
                    <?php
                }
                action_authorize_wp_footer();

            }

            /*
                * Custom CSS and JS, in most cases required only when you decided to go with a custom credit card form
                */
            public function payment_scripts()
            {
                // we need JavaScript to process a token only on cart/checkout pages, right?
                if (!is_cart() && !is_checkout()) {
                    return;
                }

                // if our payment gateway is disabled, we do not have to enqueue JS too
                if ('no' === $this->enabled) {
                    return;
                }
                wp_register_style('mecom_authorize_card_styles', plugins_url('/assets/card/card.css', __FILE__), array(), false, false);
                wp_enqueue_style('mecom_authorize_card_styles');

                wp_register_style('mecom_authorize_styles', plugins_url('assets/css/styles.css', __FILE__), [], OPT_MECOM_AUTHORIZE_VERSION);
                wp_enqueue_style('mecom_authorize_styles');

                wp_register_script('mecom_authorize_js', plugins_url('assets/js/checkout_hook.js', __FILE__), array('jquery'), OPT_MECOM_AUTHORIZE_VERSION, true);
                wp_enqueue_script('mecom_authorize_js');
            }

            /*
                * Fields validation, more in Step 5
                */
            public function validate_fields()
            {

//                if (empty($_POST['billing_first_name'])) {
//                    wc_add_notice('First name is required!', 'error');
//                    return false;
//                }
//                if (empty($_POST['mecom_ccNumber'])) {
//                    wc_add_notice('Card Number is required!', 'error');
//                    return false;
//                }
//                if (empty($_POST['mecom_ccExpirationDate'])) {
//                    wc_add_notice('Expiry Date is required!', 'error');
//                    return false;
//                }
//                if (empty($_POST['mecom_CCV'])) {
//                    wc_add_notice('CVC is required!', 'error');
//                    return false;
//                }
                return true;
            }

            private function getActivateProxyUrl()
            {
                $activatedProxy = get_option(Opt_Mecom_Authorize_Activated_Proxy, null);
                return empty($activatedProxy["url"]) ? null : $activatedProxy["url"] . '/index.php';
            }

            /*
                * We're processing the payments here, everything about it is in Step 5
                */
            public function getProductTitle($productTitle)
            {
                switch ($this->productTitleSetting) {
                    case 'user_define':
                        return $this->userDefineProductTitle;
                    case 'keep_original':
                        return $productTitle;
                    case 'last_word':
                    default:
                        return strrchr($productTitle, ' ');
                }
            }

            public function get_number_of_decimal_digits()
            {
                return $this->is_currency_supports_zero_decimal() ? 0 : 2;
            }

            public function is_currency_supports_zero_decimal()
            {
                return in_array(get_woocommerce_currency(), array('HUF', 'JPY', 'TWD'));
            }

            public function process_payment($order_id)
            {
                global $woocommerce;
                // we need it to get any order details
                $order = wc_get_order($order_id);
                $activatedProxy = findActivatedProxyDataByIdAuthorize(get_option(Opt_Mecom_Authorize_Proxies, []), WC()->session->get('mecom-authorize-proxy-active-id'));
                if (!$activatedProxy) {
                    csAuthorizeErrorLog("Can't find activated proxy!\n");
                    wc_add_notice('We cannot accept any payments right now. Please comeback to try tomorrow or select other payment methods if available.', 'error');
                    return [
                        'result' => 'failure',
                        'reload' => true
                    ];
                }

                $shippingName = $order->get_shipping_first_name() . " " . $order->get_shipping_last_name();
                $shippingAddress1 = $order->get_shipping_address_1();
                $shippingAddress2 = $order->get_shipping_address_2();
                $shippingCity = $order->get_shipping_city();
                $shippingCountry = $order->get_shipping_country();
                $shippingPostCode = $order->get_shipping_postcode();
                $shippingState = $order->get_shipping_state();
                $shippingCompany = $order->get_shipping_company();

                // Billing
                $billingName = $order->get_billing_first_name() . " " . $order->get_billing_last_name();
                $billingAddress1 = $order->get_billing_address_1();
                $billingAddress2 = $order->get_billing_address_2();
                $billingCity = $order->get_billing_city();
                $billingCountry = $order->get_billing_country();
                $billingPostCode = $order->get_billing_postcode();
                $billingState = $order->get_billing_state();
                $billingCompany = $order->get_billing_company();

                $shippingName = (empty($order->get_shipping_first_name()) && empty($order->get_shipping_last_name())) ? $billingName : $shippingName;
                $shippingAddress1 = empty($shippingAddress1) ? $billingAddress1 : $shippingAddress1;
                $shippingAddress2 = empty($shippingAddress2) ? $billingAddress2 : $shippingAddress2;
                $shippingCity = empty($shippingCity) ? $billingCity : $shippingCity;
                $shippingCountry = empty($shippingCountry) ? $billingCountry : $shippingCountry;
                $shippingPostCode = empty($shippingPostCode) ? $billingPostCode : $shippingPostCode;
                $shippingState = empty($shippingState) ? $billingState : $shippingState;
                $shippingCompany = empty($shippingCompany) ? $billingCompany : $shippingCompany;


                // Log processing proxyUrl
                $order->add_order_note(sprintf(__('Starting checkout with Authorize.net proxy %s', 'mecom'), $activatedProxy['url']), 0, false);

                $order_items = $order->get_items();
                $productNameArr = [];
                $productIdArr = [];
                foreach ( $order_items as $it ) {
                    $product = wc_get_product( $it->get_product_id() );
                    $product_name = $this->getProductTitle( $product->get_name() );
                    $item_quantity = $it->get_quantity(); // Get the item quantity
                    $productNameArr[] = $product_name . ' x ' . $item_quantity;
                    $productIdArr[] = $it->get_product_id();
                }
                $paymentIntentIdRequest = null;
                if (!empty($order->get_transaction_id())) {
                    $proxyProcessingUrl = get_post_meta($order_id, METAKEY_AUTHORIZE_PROXY_URL, true);
                    if ($proxyProcessingUrl) {
                        if (WC()->session->get('mecom-authorize-proxy-active-url') == $proxyProcessingUrl) {
                            $paymentIntentIdRequest = $order->get_transaction_id();
                        }
                    } else {
                        $paymentIntentIdRequest = $order->get_transaction_id();
                    }
                }
                $response = wp_remote_post($activatedProxy['url'] . '?mecom-authorize-make-payment=1&order_id=' . $order->get_id(), [
                    'sslverify' => false,
                    'timeout' => 5 * 60,
                    'headers' => [
                        'Content-Type' => 'application/json',
                    ],
                    'body' => json_encode([
                        'customer_ip' => $order->get_customer_ip_address(),
                        'payment_intent' => $paymentIntentIdRequest,
                        'payment_dataDescriptor' => $_POST['mecom-authorize-payment-dataDescriptor'],
                        'payment_dataValue' => $_POST['mecom-authorize-payment-dataValue'],
                        'order_id' => $order->get_id(),
                        'order_invoice' => $this->invoice_prefix . $order->get_order_number(),
                        'order_group_item' => [
                             'id' => implode( ", ", $productIdArr ),
                             'name' => implode( ", ", $productNameArr ),
                             'quantity' => 1,
                             'unit_price' => $order->get_total()
                        ],
                        'statement_descriptor' => $this->get_option('statement_descriptor'),
                        'merchant_site' => get_home_url(),
                        'amount' => $order->get_total(),
                        'customer_zipcode' => $billingPostCode,
                        'customer_email' => $order->get_billing_email(),
                        'currency' => $order->get_currency(),
                        'name' => $order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
                        'shipping' => [
                            'first_name' => empty($order->get_shipping_first_name()) ? $order->get_billing_first_name() : $order->get_shipping_first_name(),
                            'last_name' => empty($order->get_shipping_last_name()) ? $order->get_billing_last_name() : $order->get_shipping_last_name(),
                            'phone' => method_exists($order, 'get_shipping_phone') && $order->get_shipping_phone() ? $order->get_shipping_phone() : $order->get_billing_phone(),
                            'city' => $shippingCity,
                            'country' => $shippingCountry,
                            'line1' => $shippingAddress1,
                            'line2' => $shippingAddress2,
                            'postal_code' => $shippingPostCode,
                            'state' => $shippingState,
                            'company' => $shippingCompany,
                        ],
                        'billing' => [
                            'first_name' => $order->get_billing_first_name(),
                            'last_name' => $order->get_billing_last_name(),
                            'phone' => $order->get_billing_phone(),
                            'city' => $billingCity,
                            'country' => $billingCountry,
                            'line1' => $billingAddress1,
                            'line2' => $billingAddress2,
                            'postal_code' => $billingPostCode,
                            'state' => $billingState,
                            'company' => $billingCompany,
                        ]
                    ])
                ]);
                update_post_meta($order->get_id(), METAKEY_AUTHORIZE_PROXY_URL, $activatedProxy['url']);
                update_post_meta($order->get_id(), METAKEY_AUTHORIZE_PROXY_ID, $activatedProxy['id']);
                if (is_wp_error($response) || 200 !== wp_remote_retrieve_response_code($response)) {
                    csAuthorizeErrorLog($response, 'Authorize.net request error');
                    wc_add_notice('We cannot process your payment right now, please try another payment method.', 'error');
                    return false;
                }
                $body = wp_remote_retrieve_body($response);
                $body = json_decode($body);
                if ($body->status === 'success') {
                    $order->payment_complete();
                    $order->reduce_order_stock();

                    //Save the processed proxy for this order (using for refund later)
                    $order->add_order_note(sprintf(__('Authorize.net charged by proxy %s', 'mecom'), $activatedProxy['url']), 0, false);
                    // some notes to customer (replace true with false to make it private)
                    $order->add_order_note(sprintf(__('Authorize.net Checkout charge complete (Payment ID: %s)', 'mecom'), $body->transaction_id));

                    if (isEnabledAmountRotationAuthorize()) {
                        performProxyAmountRotationAuthorize($order->get_total());
                        updateRotationAmountAuthorize($activatedProxy['id'], $order->get_total());
                    }

                    update_post_meta($order->get_id(), '_transaction_id', $body->transaction_id);
                    update_post_meta($order->get_id(), METAKEY_CS_AUTHORIZE_CARD_NUMBER, $_POST['mecom-authorize-payment-cardNumber']);
                    update_post_meta($order->get_id(), METAKEY_CS_AUTHORIZE_CARD_EXPIRE, $_POST['mecom-authorize-payment-expire']);
                    // Empty cart
                    $woocommerce->cart->empty_cart();

                    return [
                        'result' => 'success',
                        'redirect' => $order->get_checkout_order_received_url()
                    ];
                } else {
                    csAuthorizeErrorLog($response, 'Authorize.net request payment error');
                    // Empty cart
                    $order->update_status('failed');
                    if ($body->code === 'domain_whitelist_not_allow') {
                        $order->add_order_note(sprintf(__('Authorize.net charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            'Domain whitelist is required'
                        ));
                    } else if ($body->code === 'customer_zipcode_not_allow') {
                        $order->add_order_note(sprintf(__('Authorize.net charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Customer's zipcode is blacklisted"
                        ));
                        wc_add_notice('The selected payment method is suspended, Please contact merchant for more information.', 'error');
                        return false;
                    } else if ($body->code === 'customer_email_not_allow') {
                        $order->add_order_note(sprintf(__('Authorize.net charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Customer's email is blacklisted"
                        ));
                        wc_add_notice('We cannot process your payment right now. Please try again with another payment method.', 'error');
                        return false;
                    } else if ($body->code === 'states_cities_not_allow') {
                        $order->add_order_note(sprintf(__('Authorize.net charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Customer's State and City is blacklisted"
                        ));
                        wc_add_notice('Sorry, Your sellected products are not available to purchase due to our policy violation.', 'error');
                        return false;
                    } else if ($body->code === 'order_total_not_allow') {
                        $order->add_order_note(sprintf(__('Authorize.net charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Order value exceeds Authorize.net capability"
                        ));
                        wc_add_notice('We cannot process your payment right now. Please try again with another payment method.', 'error');
                        return false;
                    } else if ($body->code === 'require_billing_address') {
                        $order->add_order_note(sprintf(__('Authorize.net charged ERROR by proxy %s, ERROR message: %s', 'mecom'),
                            $activatedProxy['url'],
                            "Require Billing Address"
                        ));
                        wc_add_notice('billing address required.', 'error');
                        return false;
                    } else {
                        $order->add_order_note(sprintf(__('Authorize.net charged ERROR by proxy %s, ERROR message: %s, Transaction ID: %s', 'mecom'),
                            $activatedProxy['url'],
                            $body->message,
                            $body->transaction_id
                        ));
                    }
                    wc_add_notice('We cannot process your payment right now, please try another payment method.[2]', 'error');
                    return false;
                }
            }

            /**
             * Process refund.
             *
             * @param int $order_id Order ID
             * @param float $amount Order amount
             * @param string $reason Refund reason
             *
             * @return boolean True or false based on success, or a WP_Error object.
             */
            public function process_refund($order_id, $amount = null, $reason = '')
            {
                $order = wc_get_order($order_id);
                if (0 == $amount || null == $amount) {
                    return new WP_Error('authorize_refund_error', __('Refund Error: You need to specify a refund amount.', 'mecom-authorize-gateway'));
                }

                try {
                    $this->refund_order($order, $order_id, $amount, "", $reason);
                    $order->add_order_note(sprintf(__('Authorize.net refund completed; transaction ID = %s', 'mecom-authorize-gateway'), $order->get_transaction_id()));
                    return true;
                } catch (Exception $e) {
                    csAuthorizeErrorLog($e->getMessage(), 'Authorize.net process_refund error');
                    return new WP_Error('authorize_refund_error', $e->getMessage());
                }
            }

            private function refund_order($order, $order_id, $amount, $refundType, $reason)
            {
                // add refund params
                $params['transaction_id'] = $order->get_transaction_id();
                $params['amount'] = $amount;
                $params['reason'] = $reason;
                $params["merchant_site"] = get_home_url();
                $params['currency'] = $order->get_currency();
                $params['card_number'] = get_post_meta($order_id, METAKEY_CS_AUTHORIZE_CARD_NUMBER, true);
                $params['card_expire'] = get_post_meta($order_id, METAKEY_CS_AUTHORIZE_CARD_EXPIRE, true);

                //Get the proxy url when this order was made

                $proxyUrl = get_post_meta($order_id, METAKEY_AUTHORIZE_PROXY_URL, true);

                // do API call
                $url = $proxyUrl . "?mecom-authorize-refund=1&order_id=$order_id&" . http_build_query($params);

                $request = wp_remote_get($url, [
                   'sslverify' => false
                ]);

                $notice = 'There is an error when process this payment, please contact us for more support or you can try to use Authorize.net!';
                if (is_wp_error($request)) {
                    wc_add_notice($notice, 'error');
                    $order->add_order_note(sprintf(__('Failed refund by Authorize.net! Debug proxy %s', 'mecom-authorize-gateway'), $url));
                    throw new Exception($notice);
                }

                $body = wp_remote_retrieve_body($request);
                $result = json_decode($body);
                (new WC_Logger())->log('debug', json_encode($body));
                if (isset($result->status) && $result->status == "success") {
                    return [
                        'status' => $result->status,
                    ];
                } else if ($result->status == "error") {
                    wc_add_notice($result->message, 'error');
                    $order->add_order_note(sprintf(__('Failed refund by Authorize.net! Debug proxy %s, ERROR: %s', 'mecom-authorize-gateway'), $url, $result->message));
                    throw new Exception($result->message);
                } else {
                    wc_add_notice($notice, 'error');
                    $order->add_order_note(sprintf(__('Failed refund by Authorize.net! Debug proxy %s', 'mecom-authorize-gateway'), $url));
                    throw new Exception($notice);
                }
            }

            /**
             * Get Authorize.net amount to pay
             *
             * @param float $total Amount due.
             * @param string $currency Accepted currency.
             *
             * @return float|int
             */
            public function get_authorize_amount($total, $currency = '')
            {
                if (!$currency) {
                    $currency = get_woocommerce_currency();
                }

                if (in_array(strtolower($currency), $this->no_decimal_currencies())) {
                    return absint($total);
                } else {
                    return absint(wc_format_decimal(((float)$total), wc_get_price_decimals())); // In cents.
                }
            }

            /**
             * List of currencies supported by Authorize.net that has no decimals.
             *
             * @return array $currencies
             */
            public function no_decimal_currencies()
            {
                return array(
                    'bif', // Burundian Franc
                    'djf', // Djiboutian Franc
                    'jpy', // Japanese Yen
                    'krw', // South Korean Won
                    'pyg', // Paraguayan Guaraní
                    'vnd', // Vietnamese Đồng
                    'xaf', // Central African Cfa Franc
                    'xpf', // Cfp Franc
                    'clp', // Chilean Peso
                    'gnf', // Guinean Franc
                    'kmf', // Comorian Franc
                    'mga', // Malagasy Ariary
                    'rwf', // Rwandan Franc
                    'vuv', // Vanuatu Vatu
                    'xof', // West African Cfa Franc
                );
            }

            /*
            * In case you need a webhook, like Authorize.net IPN etc
            */
            public function webhook()
            {
                // $order = wc_get_order( $_GET['id'] );
                // $order->payment_complete();
                // $order->reduce_order_stock();

                // update_option('webhook_debug', $_GET);
            }
        }
    endif;
}

register_deactivation_hook(__FILE__, 'cs_authorize_plugin_deactivation');

function cs_authorize_plugin_deactivation()
{
    wp_clear_scheduled_hook('mecom_gateway_authorize_daily');
    wp_clear_scheduled_hook('mecom_gateway_authorize_rotation');
}

//add_action('wp_footer', 'action_authorize_wp_footer', 10, 1);
function action_authorize_wp_footer()
{
    if (is_checkout()) {
        $gateways = WC()->payment_gateways->get_available_payment_gateways();
        if (isset($gateways['mecom_authorize']->enabled) && $gateways['mecom_authorize']->enabled == 'yes') {
            echo '<div id="cs-authorize-loader">
                  <div class="cs-authorize-spinnerWithLockIcon cs-authorize-spinner" aria-busy="true">
                      <p>We\'re processing your payment...<br/>Please <b>DO NOT</b> close this page!</p>
                  </div>
            </div>';
        }
    }
}

add_action('wp_head', 'action_authorize_wp_head', 10, 1);

function action_authorize_wp_head()
{
    if (is_checkout()) {
        $gateways = WC()->payment_gateways->get_available_payment_gateways();
        if (isset($gateways['mecom_authorize']->enabled) && $gateways['mecom_authorize']->enabled == 'yes') {
            if (isset($_GET['pay_for_order']) && get_query_var('order-pay')) {
                $orderIdProcessing = get_query_var('order-pay');
            } else {
                $orderIdProcessing = WC()->session->get('order_awaiting_payment');
            }
            if (!empty($orderIdProcessing)) {
                $proxyProcessingId = get_post_meta($orderIdProcessing, METAKEY_AUTHORIZE_PROXY_ID, true);
                $proxyProcessing = findActivatedProxyDataByIdAuthorize(get_option(Opt_Mecom_Authorize_Proxies, []), $proxyProcessingId);
            }
            
            if(empty($proxyProcessing)) {
                $proxyProcessing = get_option(Opt_Mecom_Authorize_Activated_Proxy, null);
                if (isEnabledAmountRotationAuthorize()) {
                    $proxyProcessing = getNextProxyAmountRotationAuthorize(WC()->cart->get_total(false));
                }
            }
            if (empty($proxyProcessing)) {
                csAuthorizeErrorLog([
                    WC()->session->get('order_awaiting_payment'),
                    get_query_var('order-pay'),
                    get_option(Opt_Mecom_Authorize_Proxies, []),
                ], 'Can not find authorize.net proxy for charge!');
                return;
            }
            WC()->session->set('mecom-authorize-proxy-active-id', $proxyProcessing['id']);
            WC()->session->set('mecom-authorize-proxy-active-url', $proxyProcessing['url']);

            echo '<link rel="preload" href="' . $proxyProcessing['url'] . '?mecom-authorize-get-payment-form=1" as="document">';
        }
    }
}
