<?php

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

const OPT_MECOM_AUTHORIZE_VERSION = '1.0.11';
const Opt_Mecom_Authorize_Proxies = 'Opt_Mecom_Authorize_Proxies';
const Opt_Mecom_Authorize_Activated_Proxy = 'Opt_Mecom_Authorize_Activated_Proxy';
const OPT_MECOM_AUTHORIZE_ROTATION_METHOD =  'OPT_MECOM_AUTHORIZE_ROTATION_METHOD';
const OPT_MECOM_AUTHORIZE_UNUSED_PROXIES = 'OPT_MECOM_AUTHORIZE_UNUSED_PROXIES';
const OPT_MECOM_AUTHORIZE_CURRENT_ROTATION_VALUE = 'OPT_MECOM_AUTHORIZE_CURRENT_ROTATION_VALUE';
const MetaKey_Authorize_Proxy_Url = '_mecom_authorize_proxy_url';
const OPT_MECOM_AUTHORIZE_LAST_TIME_RESET_PAID_AMOUNT = 'OPT_MECOM_AUTHORIZE_LAST_TIME_RESET_PAID_AMOUNT';

const METAKEY_AUTHORIZE_PROXY_URL          = '_mecom_authorize_proxy_url';
const METAKEY_AUTHORIZE_PROXY_ID          = '_mecom_authorize_proxy_id';

const MECOM_AUTHORIZE_BY_TIME                       = "by_time";
const MECOM_AUTHORIZE_BY_AMOUNT                     = "by_amount";

const METAKEY_CS_AUTHORIZE_FEE      = '_cs_authorize_fee';
const METAKEY_CS_AUTHORIZE_PAYOUT   = '_cs_authorize_payout';
const METAKEY_CS_AUTHORIZE_CURRENCY = '_cs_authorize_currency';
const METAKEY_CS_AUTHORIZE_CARD_NUMBER = '_cs_authorize_card_number';
const METAKEY_CS_AUTHORIZE_CARD_EXPIRE = '_cs_authorize_card_expire';

// true: order currency
// false: authorize currency
const MECOM_AUTHORIZE_FEE_DISPLAY_ORDER_CURRENCY = true;

function resetPaidAmountIfNeedAuthorize() {
    $lastTimeReset = get_option(OPT_MECOM_AUTHORIZE_LAST_TIME_RESET_PAID_AMOUNT, null);
    $proxies = get_option(Opt_Mecom_Authorize_Proxies, []);
    if (empty($proxies)) {
        return [];
    }
    // Reset
    if (empty($lastTimeReset) || date('Y-m-d') > $lastTimeReset) {
        return resetPaidAmountAuthorize($proxies);
    }
    return $proxies;
}

function findActivatedProxyDataByIdAuthorize($proxies, $activatedProxyId) {
    foreach ($proxies as $proxy) {
        if ($proxy['id'] === $activatedProxyId) {
            return $proxy;
        }
    }
    return null;
}

function getNextProxyAmountRotationAuthorize($orderTotal) {
    $proxies = get_option(Opt_Mecom_Authorize_Proxies, []);
    if (empty($proxies)) {
        return null;
    }
    $activatedProxy = get_option( Opt_Mecom_Authorize_Activated_Proxy, null );
    if (empty($activatedProxy)) {
        csAuthorizeErrorLog($proxies, "Activated proxy not found! Use the first proxy of rotation list");
        $activatedProxy = $proxies[0];
    }
    $activatedProxy = findActivatedProxyDataByIdAuthorize($proxies, $activatedProxy['id']);

    // Need to rotate proxy
    $isCurrentProxyMatched = false;
    foreach ($proxies as $proxy) {
        if($activatedProxy['id'] == $proxy['id'] && !$isCurrentProxyMatched) {
            $isCurrentProxyMatched = true;
            continue;
        }
        if($isCurrentProxyMatched && doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            return $proxy;
        }
    }
    foreach ($proxies as $proxy) {
        if(doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            return $proxy;
        }
    }
    return null;
}

function performProxyAmountRotationAuthorize($orderTotal) {
    $proxies = resetPaidAmountIfNeedAuthorize();
    if (empty($proxies)) {
        return null;
    }
    $activatedProxy = get_option( Opt_Mecom_Authorize_Activated_Proxy, null );
    if (empty($activatedProxy)) {
        csAuthorizeErrorLog("Activated proxy not found! Use the first proxy of rotation list");
        $activatedProxy = $proxies[0];
        update_option(Opt_Mecom_Authorize_Activated_Proxy, $activatedProxy, true);
        logAuthorizeRotation(MECOM_AUTHORIZE_BY_AMOUNT, $activatedProxy, "Auto");
    }
    $activatedProxy = findActivatedProxyDataByIdAuthorize($proxies, $activatedProxy['id']);

    // Need to rotate proxy
    $isCurrentProxyMatched = false;
    foreach ($proxies as $proxy) {
        if($activatedProxy['id'] == $proxy['id'] && !$isCurrentProxyMatched) {
            $isCurrentProxyMatched = true;
            continue;
        }
        if($isCurrentProxyMatched && doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            $activatedProxy = $proxy;
            update_option(Opt_Mecom_Authorize_Activated_Proxy, $activatedProxy, true);
            logAuthorizeRotation(MECOM_AUTHORIZE_BY_AMOUNT, $activatedProxy, "Auto");
            return $activatedProxy;
        }
    }
    foreach ($proxies as $proxy) {
        if(doubleval( $proxy['paid_amount'] ) + doubleval( $orderTotal ) < doubleval($proxy['amount'])) {
            $activatedProxy = $proxy;
            update_option(Opt_Mecom_Authorize_Activated_Proxy, $activatedProxy, true);
            logAuthorizeRotation(MECOM_AUTHORIZE_BY_AMOUNT, $activatedProxy, "Auto");
            return $activatedProxy;
        }
    }
    return null;
}

function resetPaidAmountAuthorize($proxies = null)
{
    if (empty($proxies)) {
        $proxies = get_option( Opt_Mecom_Authorize_Proxies, [] );
    }
    if (empty($proxies)) return [];
    // Reset
    $newProxies = array_map(function ($proxy) {
        $proxy['paid_amount'] = 0;
        return $proxy;
    }, $proxies);
    $unusedProxies = get_option(OPT_MECOM_AUTHORIZE_UNUSED_PROXIES, []);
    if (empty($unusedProxies)) {
        $newUnusedProxies = [];
    } else {
        $newUnusedProxies = array_map(function ($unusedProxy) {
            $unusedProxy['paid_amount'] = 0;
            return $unusedProxy;
        }, $unusedProxies);
    }
    update_option(Opt_Mecom_Authorize_Proxies, $newProxies, true);
    update_option(OPT_MECOM_AUTHORIZE_UNUSED_PROXIES, $newUnusedProxies, true);
    update_option(OPT_MECOM_AUTHORIZE_LAST_TIME_RESET_PAID_AMOUNT, date('Y-m-d'), true);
    return $newProxies;
}

function logAuthorizeRotation($rotationMethod, $proxy, $type)
{
    $mecom_log_dir = wp_get_upload_dir()["basedir"] . '/mecom';
    if (!is_dir($mecom_log_dir)) {
        mkdir($mecom_log_dir, 0777, true);
    }
    $logFilePath = $mecom_log_dir . '/' . getAuthorizeLogFileName($rotationMethod);
    $currentDateTime = date('Y-m-d H:i:s');
    $rotationValue = $rotationMethod === MECOM_AUTHORIZE_BY_TIME ? $proxy['timestamp'] : ($proxy['paid_amount'] . '/' . $proxy['amount']);
    $content = "{$currentDateTime} - {$proxy['url']} , {$rotationValue} - {$type}\n";

    $fp = fopen($logFilePath, 'a');
    fwrite($fp, $content);
    fclose($fp);
}

function getAuthorizeLogFileName($rotationMethod) {
    if ( $rotationMethod === MECOM_AUTHORIZE_BY_TIME) {
        return 'mecom_authorize_rotation_time_log.txt';
    } else if ( $rotationMethod === MECOM_AUTHORIZE_BY_AMOUNT) {
        return 'mecom_authorize_rotation_amount_log.txt';
    }
}

function loadAuthorizeLogs() {
    $mecom_log_dir = wp_get_upload_dir()["basedir"].'/mecom';
    $logFilePath = $mecom_log_dir.'/authorize_rotation_log.txt';
    if (file_exists($logFilePath)) {
        $array = explode("\n", file_get_contents($logFilePath));   
    }else{
        $array = array();
    }
    return $array;
}

function isEnabledAmountRotationAuthorize() {
    return MECOM_AUTHORIZE_BY_AMOUNT === get_option(OPT_MECOM_AUTHORIZE_ROTATION_METHOD, MECOM_AUTHORIZE_BY_TIME);
}

function updateRotationAmountAuthorize($processedProxyId, $orderTotal) {
    $proxies = get_option(Opt_Mecom_Authorize_Proxies, []);
    foreach ($proxies as $key => $proxy) {
        if ($proxy['id'] === $processedProxyId) {
            $proxies[$key]['paid_amount'] = doubleval($proxy['paid_amount']) + doubleval( $orderTotal );
            break;
        }
    }
    return update_option(Opt_Mecom_Authorize_Proxies, $proxies, true);
}

function hasPayableProxyAuthorize($cartTotal) {
    $proxies = resetPaidAmountIfNeedAuthorize();
    if (empty($proxies)) {
        return false;
    }

    foreach ($proxies as $proxy) {
        if(doubleval( $proxy['paid_amount'] ) + doubleval( $cartTotal ) < doubleval($proxy['amount'])) {
            return true;
        }
    }
    return false;
}

function authorizeLog($error, $message = '')
{
    $logger = wc_get_logger();
    if (is_array($error) || is_object($error)) {
        $error = print_r($error, true);
    }
    $error = '\n--------------------- ' . $message . ' ---------------------\n'
        . $error .
        '\n------------------------------------------\n';
    if (empty($logger)) {
        error_log($error);
    } else {
        $logger->debug($error, ['source' => 'cardshield-gateway-authorize']);
    }
}

function csAuthorizeErrorLog($data, $message = '')
{
    $trace = debug_backtrace();
    $dataLogString = csAuthorizeHandleDataLog($data, $message) 
        . print_r([$trace[1]['class'], $trace[1]['function'], $trace[1]['args']], true);
    if (!$logger = wc_get_logger()) {
        error_log($dataLogString);
    } else {
        $logger->debug($dataLogString, ['source' => 'cardshield-gateway-authorize-ERROR']);
    }
}

function csAuthorizeDebugLog($data, $message = '')
{
    $trace = debug_backtrace();
    $dataLogString = csAuthorizeHandleDataLog($data, $message) 
        . print_r([$trace[1]['class'], $trace[1]['function'], $trace[1]['args']], true);
    if (!$logger = wc_get_logger()) {
        error_log($dataLogString);
    } else {
        $logger->debug($dataLogString, ['source' => 'cardshield-gateway-authorize-INFO']);
    }
}

function csAuthorizeHandleDataLog($data, $message = '') {
    try {
        if (is_array($data) || is_object($data)) {
            $dataLog = print_r($data, true);
        } else {
            $dataLog = (string)$data;
        }
    } catch (\Exception $e) {
        $dataLog = 'csAuthorizeLog ERROR: ' . $e->getMessage();
    }
    return '\n--------------------- ' . $message . ' ---------------------\n'
        . $dataLog;
}
